@@ -1,7 +1,7 @@
 #!/bin/bash
 # Copyright 1999-2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: ebuild.sh 7404 2007-07-27 18:34:54Z zmedico $
+# $Id: ebuild.sh 8231 2007-10-22 20:33:08Z zmedico $
 
 PORTAGE_BIN_PATH="${PORTAGE_BIN_PATH:-/usr/lib/portage/bin}"
 PORTAGE_PYM_PATH="${PORTAGE_PYM_PATH:-/usr/lib/portage/pym}"
@@ -284,23 +284,56 @@ register_die_hook() {
 diefunc() {
 	local funcname="$1" lineno="$2" exitcode="$3"
 	shift 3
+
+	local n filespacing=0 linespacing=0
+	# setup spacing to make output easier to read
+	for ((n = ${#FUNCNAME[@]} - 1; n >= 0; --n)); do
+		sourcefile=${BASH_SOURCE[${n}]} sourcefile=${sourcefile##*/}
+		lineno=${BASH_LINENO[${n}]}
+		((filespacing < ${#sourcefile})) && filespacing=${#sourcefile}
+		((linespacing < ${#lineno}))     && linespacing=${#lineno}
+	done
+
 	eerror
 	eerror "ERROR: $CATEGORY/$PF failed."
-	dump_trace 2
-	eerror "  ${BASH_SOURCE[1]##*/}, line ${BASH_LINENO[0]}:   Called die"
+	dump_trace 2 ${filespacing} ${linespacing}
+	eerror "  $(printf "%${filespacing}s" "${BASH_SOURCE[1]##*/}"), line $(printf "%${linespacing}s" "${BASH_LINENO[0]}"):  Called die"
+	eerror "The specific snippet of code:"
+	# This scans the file that called die and prints out the logic that
+	# ended in the call to die.  This really only handles lines that end
+	# with '|| die' and any preceding lines with line continuations (\).
+	# This tends to be the most common usage though, so let's do it.
+	# Due to the usage of appending to the hold space (even when empty),
+	# we always end up with the first line being a blank (thus the 2nd sed).
+	sed -n \
+		-e "# When we get to the line that failed, append it to the
+		    # hold space, move the hold space to the pattern space,
+		    # then print out the pattern space and quit immediately
+		    ${BASH_LINENO[0]}{H;g;p;q}" \
+		-e '# If this line ends with a line continuation, append it
+		    # to the hold space
+		    /\\$/H' \
+		-e '# If this line does not end with a line continuation,
+		    # erase the line and set the hold buffer to it (thus
+		    # erasing the hold buffer in the process)
+		    /[^\]$/{s:^.*$::;h}' \
+		${BASH_SOURCE[1]} \
+		| sed -e '1d' -e 's:^:RETAIN-LEADING-SPACE:' \
+		| while read -r n ; do eerror "  ${n#RETAIN-LEADING-SPACE}" ; done
+	eerror " The die message:"
+	eerror "  ${*:-(no error message)}"
 	eerror
-	eerror "${*:-(no error message)}"
 	eerror "If you need support, post the topmost build error, and the call stack if relevant."
-	[ -n "${PORTAGE_LOG_FILE}" ] && \
-		eerror "A complete build log is located at '${PORTAGE_LOG_FILE}'."
-	if [ -n "${EBUILD_OVERLAY_ECLASSES}" ] ; then
+	[[ -n ${PORTAGE_LOG_FILE} ]] \
+		&& eerror "A complete build log is located at '${PORTAGE_LOG_FILE}'."
+	if [[ -n ${EBUILD_OVERLAY_ECLASSES} ]] ; then
 		eerror "This ebuild used the following eclasses from overlays:"
 		local x
 		for x in ${EBUILD_OVERLAY_ECLASSES} ; do
 			eerror "  ${x}"
 		done
 	fi
-	if [ "${EBUILD#${PORTDIR}/}" == "${EBUILD}" ] ; then
+	if [[ "${EBUILD#${PORTDIR}/}" == "${EBUILD}" ]] ; then
 		local overlay=${EBUILD%/*}
 		overlay=${overlay%/*}
 		overlay=${overlay%/*}
@@ -308,7 +341,7 @@ diefunc() {
 	fi
 	eerror
 
-	if [ "${EBUILD_PHASE/depend}" == "${EBUILD_PHASE}" ]; then
+	if [[ "${EBUILD_PHASE/depend}" == "${EBUILD_PHASE}" ]] ; then
 		local x
 		for x in $EBUILD_DEATH_HOOKS; do
 			${x} "$@" >&2 1>&2
@@ -322,19 +355,16 @@ diefunc() {
 
 shopt -s extdebug &> /dev/null
 
-# usage- first arg is the number of funcs on the stack to ignore.
-# defaults to 1 (ignoring dump_trace)
+# dump_trace([number of funcs on stack to skip],
+#            [whitespacing for filenames],
+#            [whitespacing for line numbers])
 dump_trace() {
 	local funcname="" sourcefile="" lineno="" n e s="yes"
+	declare -i strip=${1:-1}
+	local filespacing=$2 linespacing=$3
 
-	declare -i strip=1
-
-	if [[ -n $1 ]]; then
-		strip=$(( $1 ))
-	fi
-	
 	eerror "Call stack:"
-	for (( n = ${#FUNCNAME[@]} - 1, p = ${#BASH_ARGV[@]} ; n > $strip ; n-- )) ; do
+	for (( n = ${#FUNCNAME[@]} - 1, p = ${#BASH_ARGV[@]} ; n > ${strip} ; n-- )) ; do
 		funcname=${FUNCNAME[${n} - 1]}
 		sourcefile=$(basename ${BASH_SOURCE[${n}]})
 		lineno=${BASH_LINENO[${n} - 1]}
@@ -347,7 +377,7 @@ dump_trace() {
 			done
 			(( p -= ${BASH_ARGC[${n} - 1]} ))
 		fi
-		eerror "  ${sourcefile}, line ${lineno}:   Called ${funcname}${args:+ ${args}}"
+		eerror "  $(printf "%${filespacing}s" "${sourcefile}"), line $(printf "%${linespacing}s" "${lineno}"):  Called ${funcname}${args:+ ${args}}"
 	done
 }
 
@@ -386,10 +416,13 @@ keepdir() {
 	local x
 	if [ "$1" == "-R" ] || [ "$1" == "-r" ]; then
 		shift
-		find "$@" -type d -printf "${D}/%p/.keep_${CATEGORY}_${PN}-${SLOT}\n" | tr "\n" "\0" | $XARGS -0 -n100 touch || die "Failed to recursively create .keep files"
+		find "$@" -type d -printf "${D}%p/.keep_${CATEGORY}_${PN}-${SLOT}\n" \
+			| tr "\n" "\0" | ${XARGS} -0 -n100 touch || \
+			die "Failed to recursively create .keep files"
 	else
 		for x in "$@"; do
-			touch "${D}/${x}/.keep_${CATEGORY}_${PN}-${SLOT}" || die "Failed to create .keep in ${D}/${x}"
+			touch "${D}${x}/.keep_${CATEGORY}_${PN}-${SLOT}" || \
+				die "Failed to create .keep in ${D}${x}"
 		done
 	fi
 }
@@ -448,20 +481,28 @@ unpack() {
 				;;
 			7Z|7z)
 				local my_output
-				my_output="$(7z x -y "${srcdir}/${x}")"
+				my_output="$(7z x -y "${srcdir}${x}")"
 				if [ $? -ne 0 ]; then
 					echo "${my_output}" >&2
 					die "$myfail"
 				fi
 				;;
 			RAR|rar)
-				unrar x -idq -o+ "${srcdir}/${x}" || die "$myfail"
+				unrar x -idq -o+ "${srcdir}${x}" || die "$myfail"
 				;;
 			LHa|LHA|lha|lzh)
-				lha xfq "${srcdir}/${x}" || die "$myfail"
+				lha xfq "${srcdir}${x}" || die "$myfail"
 				;;
 			a|deb)
-				ar x "${srcdir}/${x}" || die "$myfail"
+				ar x "${srcdir}${x}" || die "$myfail"
+				;;
+			lzma)
+				if [ "${y}" == "tar" ]; then
+					lzma -dc "${srcdir}${x}" | tar xof - ${tar_opts}
+					assert "$myfail"
+				else
+					lzma -dc "${srcdir}${x}" > ${x%.*} || die "$myfail"
+				fi
 				;;
 			*)
 				vecho "unpack ${x}: file format not recognized. Ignoring."
@@ -614,21 +655,21 @@ einstall() {
 
 	if [ -f ./[mM]akefile -o -f ./GNUmakefile ] ; then
 		if [ "${PORTAGE_DEBUG}" == "1" ]; then
-			${MAKE:-make} -n prefix="${D}/usr" \
-				datadir="${D}/usr/share" \
-				infodir="${D}/usr/share/info" \
-				localstatedir="${D}/var/lib" \
-				mandir="${D}/usr/share/man" \
-				sysconfdir="${D}/etc" \
+			${MAKE:-make} -n prefix="${D}usr" \
+				datadir="${D}usr/share" \
+				infodir="${D}usr/share/info" \
+				localstatedir="${D}var/lib" \
+				mandir="${D}usr/share/man" \
+				sysconfdir="${D}etc" \
 				${LOCAL_EXTRA_EINSTALL} \
 				"$@" install
 		fi
-		${MAKE:-make} prefix="${D}/usr" \
-			datadir="${D}/usr/share" \
-			infodir="${D}/usr/share/info" \
-			localstatedir="${D}/var/lib" \
-			mandir="${D}/usr/share/man" \
-			sysconfdir="${D}/etc" \
+		${MAKE:-make} prefix="${D}usr" \
+			datadir="${D}usr/share" \
+			infodir="${D}usr/share/info" \
+			localstatedir="${D}var/lib" \
+			mandir="${D}usr/share/man" \
+			sysconfdir="${D}etc" \
 			${LOCAL_EXTRA_EINSTALL} \
 			"$@" install || die "einstall failed"
 	else
@@ -655,7 +696,9 @@ src_unpack() {
 }
 
 src_compile() {
-	if [ -x ./configure ]; then
+	if [ "${EAPI:-0}" == 0 ] ; then
+		[ -x ./configure ] && econf
+	elif [ -x "${ECONF_SOURCE:-.}/configure" ] ; then
 		econf
 	fi
 	if [ -f Makefile ] || [ -f GNUmakefile ] || [ -f makefile ]; then
@@ -977,6 +1020,8 @@ dyn_compile() {
 	if [ -d "${S}" ]; then
 		srcdir=${S}
 		cd "${S}"
+	else
+		cd "${WORKDIR}"
 	fi
 	#our custom version of libtool uses $S and $D to fix
 	#invalid paths in .la files
@@ -1035,6 +1080,8 @@ dyn_test() {
 	trap "abort_test" SIGINT SIGQUIT
 	if [ -d "${S}" ]; then
 		cd "${S}"
+	else
+		cd "${WORKDIR}"
 	fi
 	if ! hasq test $FEATURES && [ "${EBUILD_FORCE_TEST}" != "1" ]; then
 		vecho ">>> Test phase [not enabled]: ${CATEGORY}/${PF}"
@@ -1071,6 +1118,8 @@ dyn_install() {
 	mkdir "${PORTAGE_BUILDDIR}/image"
 	if [ -d "${S}" ]; then
 		cd "${S}"
+	else
+		cd "${WORKDIR}"
 	fi
 	vecho
 	vecho ">>> Install ${PF} into ${D} category ${CATEGORY}"
@@ -1187,7 +1236,7 @@ debug-print() {
 		fi
 
 		# default target
-		echo "$1" >> "${T}/eclass-debug.log"
+		echo "$1" 2>/dev/null >> "${T}/eclass-debug.log"
 		# let the portage user own/write to this file
 		chmod g+w "${T}/eclass-debug.log" &>/dev/null
 
@@ -1551,9 +1600,7 @@ then
 fi
 
 #a reasonable default for $S
-if [ "$S" = "" ]; then
-	export S=${WORKDIR}/${P}
-fi
+[[ -z ${S} ]] && export S=${WORKDIR}/${P}
 
 #wipe the interceptors.  we don't want saved.
 if hasq "depend" "${EBUILD_SH_ARGS}"; then
