@@ -1,7 +1,7 @@
 # getbinpkg.py -- Portage binary-package helper functions
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: getbinpkg.py 7026 2007-06-25 04:50:48Z zmedico $
+# $Id: getbinpkg.py 7811 2007-09-26 04:00:51Z zmedico $
 
 if not hasattr(__builtins__, "set"):
 	from sets import Set as set
@@ -298,6 +298,10 @@ def dir_get_list(baseurl,conn=None):
 
 	listing = None
 	if protocol in ["http","https"]:
+		if not address.endswith("/"):
+			# http servers can return a 400 error here
+			# if the address doesn't end with a slash.
+			address += "/"
 		page,rc,msg = make_http_request(conn,address,params,headers)
 		
 		if page:
@@ -458,13 +462,15 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 		makepickle = "/var/cache/edb/metadata.idx.most_recent"
 
 	conn,protocol,address,params,headers = create_conn(baseurl, conn)
+	out = sys.stdout
 
 	filedict = {}
 
 	try:
 		metadatafile = open("/var/cache/edb/remote_metadata.pickle")
 		metadata = cPickle.load(metadatafile)
-		sys.stderr.write("Loaded metadata pickle.\n")
+		out.write("Loaded metadata pickle.\n")
+		out.flush()
 		metadatafile.close()
 	except (cPickle.UnpicklingError, OSError, IOError, EOFError):
 		metadata = {}
@@ -505,10 +511,12 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 					sys.stderr.write("--- "+str(e)+"\n")
 					if trynum < 3:
 						sys.stderr.write("Retrying...\n")
+					sys.stderr.flush()
 					mytempfile.close()
 					continue
 				if match_in_array([mfile],suffix=".gz"):
-					sys.stderr.write("gzip'd\n")
+					out.write("gzip'd\n")
+					out.flush()
 					try:
 						import gzip
 						mytempfile.seek(0)
@@ -519,6 +527,7 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 					except Exception, e:
 						mytempfile.close()
 						sys.stderr.write("!!! Failed to use gzip: "+str(e)+"\n")
+						sys.stderr.flush()
 					mytempfile.close()
 				try:
 					metadata[baseurl]["data"] = cPickle.loads(data)
@@ -526,13 +535,15 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 					metadata[baseurl]["indexname"] = mfile
 					metadata[baseurl]["timestamp"] = int(time.time())
 					metadata[baseurl]["modified"]  = 0 # It's not, right after download.
-					sys.stderr.write("Pickle loaded.\n")
+					out.write("Pickle loaded.\n")
+					out.flush()
 					break
 				except SystemExit, e:
 					raise
 				except Exception, e:
 					sys.stderr.write("!!! Failed to read data from index: "+str(mfile)+"\n")
 					sys.stderr.write("!!! "+str(e)+"\n")
+					sys.stderr.flush()
 			try:
 				metadatafile = open("/var/cache/edb/remote_metadata.pickle", "w+")
 				cPickle.dump(metadata,metadatafile)
@@ -542,15 +553,40 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 			except Exception, e:
 				sys.stderr.write("!!! Failed to write binary metadata to disk!\n")
 				sys.stderr.write("!!! "+str(e)+"\n")
+				sys.stderr.flush()
 			break
 	# We may have metadata... now we run through the tbz2 list and check.
-	sys.stderr.write(yellow("cache miss: 'x'")+" --- "+green("cache hit: 'o'")+"\n")
+
+	class CacheStats(object):
+		from time import time
+		def __init__(self, out):
+			self.misses = 0
+			self.hits = 0
+			self.last_update = 0
+			self.out = out
+			self.min_display_latency = 0.2
+		def update(self):
+			cur_time = self.time()
+			if cur_time - self.last_update >= self.min_display_latency:
+				self.last_update = cur_time
+				self.display()
+		def display(self):
+			self.out.write("\r"+yellow("cache miss: '"+str(self.misses)+"'")+\
+				" --- "+green("cache hit: '"+str(self.hits)+"'"))
+			self.out.flush()
+
+	cache_stats = CacheStats(out)
+	have_tty = out.isatty()
+	if have_tty:
+		cache_stats.display()
 	binpkg_filenames = set()
 	for x in tbz2list:
 		x = os.path.basename(x)
 		binpkg_filenames.add(x)
 		if x not in metadata[baseurl]["data"]:
-			sys.stderr.write(yellow("x"))
+			cache_stats.misses += 1
+			if have_tty:
+				cache_stats.update()
 			metadata[baseurl]["modified"] = 1
 			myid = None
 			for retry in xrange(3):
@@ -571,8 +607,12 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 				metadata[baseurl]["data"][x] = make_metadata_dict(myid)
 			elif verbose:
 				sys.stderr.write(red("!!! Failed to retrieve metadata on: ")+str(x)+"\n")
+				sys.stderr.flush()
 		else:
-			sys.stderr.write(green("o"))
+			cache_stats.hits += 1
+			if have_tty:
+				cache_stats.update()
+	cache_stats.display()
 	# Cleanse stale cache for files that don't exist on the server anymore.
 	stale_cache = set(metadata[baseurl]["data"]).difference(binpkg_filenames)
 	if stale_cache:
@@ -581,8 +621,9 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 		metadata[baseurl]["modified"] = 1
 	del stale_cache
 	del binpkg_filenames
-	sys.stderr.write("\n")
-	
+	out.write("\n")
+	out.flush()
+
 	try:
 		if metadata[baseurl].has_key("modified") and metadata[baseurl]["modified"]:
 			metadata[baseurl]["timestamp"] = int(time.time())
@@ -598,6 +639,7 @@ def dir_get_metadata(baseurl, conn=None, chunk_size=3000, verbose=1, usingcache=
 	except Exception, e:
 		sys.stderr.write("!!! Failed to write binary metadata to disk!\n")
 		sys.stderr.write("!!! "+str(e)+"\n")
+		sys.stderr.flush()
 
 	if not keepconnection:
 		conn.close()
