@@ -1,7 +1,7 @@
 # deps.py -- Portage dependency resolution functions
 # Copyright 2003-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_dep.py 6798 2007-06-11 02:49:15Z zmedico $
+# $Id: portage_dep.py 7730 2007-09-06 17:00:37Z zmedico $
 
 
 # DEPEND SYNTAX:
@@ -543,8 +543,6 @@ def best_match_to_list(mypkg, mylist):
 			bestm  = x
 	return bestm
 
-_match_from_list_cache = {}
-
 def match_from_list(mydep, candidate_list):
 	"""
 	Searches list for entries that matches the package.
@@ -557,12 +555,6 @@ def match_from_list(mydep, candidate_list):
 	@return: A list of package atoms that match the given package atom
 	"""
 
-	global _match_from_list_cache
-	cache_key = (mydep, tuple(candidate_list))
-	mylist = _match_from_list_cache.get(cache_key, None)
-	if mylist is not None:
-		return mylist[:]
-
 	from portage_util import writemsg
 	if mydep[0] == "!":
 		mydep = mydep[1:]
@@ -666,5 +658,4 @@ def match_from_list(mydep, candidate_list):
 				continue
 			mylist.append(x)
 
-	_match_from_list_cache[cache_key] = mylist
-	return mylist[:]
+	return mylist
