@@ -1,6 +1,6 @@
 # Copyright 2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: portage_util.py 6931 2007-06-22 02:09:41Z zmedico $
+# $Id: portage_util.py 7545 2007-08-02 17:19:32Z zmedico $
 
 from portage_exception import PortageException, FileNotFound, \
        OperationNotPermitted, PermissionDenied, ReadOnlyFileSystem
@@ -852,7 +852,7 @@ def ensure_dirs(dir_path, *args, **kwargs):
 		created_dir = True
 	except OSError, oe:
 		func_call = "makedirs('%s')" % dir_path
-		if errno.EEXIST == oe.errno:
+		if oe.errno in (errno.EEXIST, errno.EISDIR):
 			pass
 		elif oe.errno == errno.EPERM:
 			raise OperationNotPermitted(func_call)
