@@ -1,7 +1,7 @@
 # test_vercmp.py -- Portage Unit Testing Functionality
 # Copyright 2006 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: test_vercmp.py 5537 2007-01-11 05:09:38Z antarus $
+# $Id: test_vercmp.py 7623 2007-08-15 22:20:11Z zmedico $
 
 from unittest import TestCase
 from portage_versions import vercmp
@@ -21,7 +21,10 @@ class VerCmpTestCase(TestCase):
 		pre < alpha < beta < rc < p -> test each of these, they are inductive (or should be..)
 		"""
 		tests = [ ( "4.0", "5.0"), ("5", "5.0"), ("1.0_pre2","1.0_p2"),
-			("1.0_alpha2", "1.0_p2"),("1.0_alpha1", "1.0_beta1"),("1.0_beta3","1.0_rc3")]
+			("1.0_alpha2", "1.0_p2"),("1.0_alpha1", "1.0_beta1"),("1.0_beta3","1.0_rc3"),
+			("1.001000000000000000001", "1.001000000000000000002"),
+			("1.00100000000", "1.0010000000000000001"),
+			("1.01", "1.1")]
 		for test in tests:
 			self.failIf( vercmp( test[0], test[1]) >= 0, msg="%s > %s? Wrong!" % (test[0],test[1]))
 	
